package com.tzld.longarticle.recommend.server.remote;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.HttpPoolFactory;
import com.tzld.longarticle.recommend.server.common.enums.recommend.FeishuRobotIdEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.util.MapBuilder;
import com.tzld.longarticle.recommend.server.util.feishu.FeishuMessageSender;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.HttpEntity;
import org.apache.http.StatusLine;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Service
@Slf4j
public class NLPRemoteService {

    @Autowired
    FeishuMessageSender feishuMessageSender;

    @Value("${nlp.chat.id}")
    private String chatId;
    @ApolloJsonValue("${score.interest.type:{}}")
    private Map<String, String> accountScoreInterestTypeMap;
    @ApolloJsonValue("${score.sim.type:{}}")
    private Map<String, String> accountScoreSimTypeMap;
    @ApolloJsonValue("${score.viewCount.filter:{}}")
    private Map<String, Integer> accountScoreViewCountFilterMap;
    @ApolloJsonValue("${score.rate:{}}")
    private Map<String, Double> accountScoreRaterMap;
    @ApolloJsonValue("${score.min.time:{}}")
    private Map<String, Long> accountScoreMinTimeMap;

    private static CloseableHttpClient client = HttpPoolFactory.nlpPool();
    private static final String scoreListUrl = "http://61.48.133.26:6060/score_list";
    private static final String backUrl = "http://47.98.136.48:6060/score_list";

    public Map<String, Double> score(String ghId, String accountName, List<Content> contentList) {
        long start = System.currentTimeMillis();
        Map<String, Double> result = new HashMap<>();
        List<String> titleList = contentList.stream().map(Content::getTitle).collect(Collectors.toList());
        String url = scoreListUrl;
        JSONObject bodyParam = new JSONObject();
        bodyParam.put("gh_id_list", Collections.singletonList(ghId));
        bodyParam.put("account_nickname_list", Collections.singletonList(accountName));
        bodyParam.put("text_list", titleList);
        bodyParam.put("interest_type", accountScoreInterestTypeMap.getOrDefault(accountName, "avg"));
        bodyParam.put("sim_type", accountScoreSimTypeMap.getOrDefault(accountName, "mean"));
        bodyParam.put("rate", accountScoreRaterMap.getOrDefault(accountName, 0.1));
        if (accountScoreViewCountFilterMap.containsKey(accountName)) {
            bodyParam.put("view_count_filter", accountScoreViewCountFilterMap.get(accountName));
        }
        if (accountScoreMinTimeMap.containsKey(accountName)) {
            bodyParam.put("min_time", accountScoreMinTimeMap.get(accountName));
        }
        int retry = 0;
        while (retry < 3) {
            retry++;
            try {
                HttpPost httpPost = new HttpPost(url);
                StringEntity stringEntity = new StringEntity(bodyParam.toJSONString(), StandardCharsets.UTF_8);
                httpPost.setHeader("Content-Type", "application/json;charset=UTF-8");
                httpPost.setEntity(stringEntity);
                CloseableHttpResponse response = client.execute(httpPost);
                StatusLine statusLine = response.getStatusLine();
                if (statusLine.getStatusCode() == 200) {
                    HttpEntity responseEntity = response.getEntity();
                    if (Objects.nonNull(responseEntity)) {
                        String responseBody = EntityUtils.toString(responseEntity, "UTF-8");
//                        log.info("scoreList 返回的数据：{}", responseBody);
                        JSONObject scoreListResponse = JSONObject.parseObject(responseBody);
                        if (Objects.nonNull(scoreListResponse)) {
                            Set<String> keySet = scoreListResponse.keySet();
                            JSONObject accountScoreList = null;
                            for (String key : keySet) {
                                if (key.equals(accountName) || key.equals(ghId)) {
                                    accountScoreList = scoreListResponse.getJSONObject(key);
                                    break;
                                }
                            }
                            if (Objects.nonNull(accountScoreList)) {
                                List<Double> scoreList = accountScoreList.getJSONArray("score_list").toJavaList(Double.class);
                                for (int i = 0; i < contentList.size(); i++) {
                                    result.put(contentList.get(i).getId(), scoreList.get(i));
                                }
                                return result;
                            }
                        }
                    }
                }
            } catch (Exception e) {
                feishuMessageSender.sendChat(chatId, "4090计算评分失败",
                        MapBuilder.<String, String>builder(new LinkedHashMap<>())
                                .put("url", url)
                                .put("账号名称", accountName)
                                .build());
                FeishuMessageSender.sendWebHookMessage(FeishuRobotIdEnum.RECOMMEND.getRobotId(),
                        "4090计算评分失败\n"
                        + "url: " + url +"\n"
                        + "账号名称: " + accountName);
                url = backUrl;
                client = HttpPoolFactory.thirtySecondPool();
                log.error("score error", e);
            }
        }
        log.info("score耗时：{}", System.currentTimeMillis() - start);
        return Collections.emptyMap();
    }

}
