package com.tzld.longarticle.recommend.server.service.cgi.impl;

import com.tzld.longarticle.recommend.server.common.enums.cgi.ReplyStrategyServiceEnum;
import com.tzld.longarticle.recommend.server.model.cgi.BucketDataParam;
import com.tzld.longarticle.recommend.server.model.cgi.ReplyBucketData;
import com.tzld.longarticle.recommend.server.service.cgi.CgiReplyService;
import com.tzld.longarticle.recommend.server.service.strategy.reply.ReplyStrategyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Map;

@Service
public class CgiReplyServiceImpl implements CgiReplyService {

    @Autowired
    private ApplicationContext applicationContext;

    private Map<String, ReplyStrategyService> strategyServiceMap;

    @PostConstruct
    public void init() {
        strategyServiceMap = applicationContext.getBeansOfType(ReplyStrategyService.class);
    }

    /**
     * 当前使用策略
     */
    private final ReplyStrategyServiceEnum useStrategy = ReplyStrategyServiceEnum.BUCKET_STRATEGY_V1;


    @Override
    public ReplyBucketData getRgiReplyData(BucketDataParam bucketDataParam) {
        for (Map.Entry<String, ReplyStrategyService> stringReplyStrategyServiceEntry : strategyServiceMap.entrySet()) {
            ReplyStrategyService replyStrategyService = stringReplyStrategyServiceEntry.getValue();
            // 使用策略层
            if (replyStrategyService.support(useStrategy)) {
                return replyStrategyService.getResult(bucketDataParam);
            }
        }
        // 无执行策略 不会走到这里
        return null;
    }
}
