package com.tzld.longarticle.recommend.server.service.recommend.filter.strategy;

import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @author dyp
 */
@Component
@Slf4j
public class ColdStartBackupFilterStrategy implements FilterStrategy {
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> contents = param.getContents();
        String[] pools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        for (Content content : contents) {
            if (pools.length > 2 && StringUtils.equals(pools[2], content.getContentPoolType())) {
                result.add(content.getId());
            }
        }
        filterResult.setContentIds(result);
        log.info("ColdStartBackupFilterStrategy cost:{}", System.currentTimeMillis() - start);
        return filterResult;
    }

}
