package com.tzld.longarticle.recommend.server.service.recommend.filter.strategy;

import cn.hutool.core.collection.CollectionUtil;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.remote.ArticleSensitiveRemoteService;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleSensitiveRepository;
import com.tzld.longarticle.recommend.server.model.entity.crawler.ArticleSensitive;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Component
@Slf4j
public class SensitiveStrategy implements FilterStrategy {

    @Autowired
    private ArticleSensitiveRemoteService articleSensitiveRemoteService;
    @Autowired
    private ArticleSensitiveRepository articleSensitiveRepository;

    @ApolloJsonValue("${UnSafeTitles:[]}")
    private static List<String> UnSafeTitles;

    private final ExecutorService pool = ThreadPoolFactory.deDuplicatePool();

    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> filterContents = new ArrayList<>();

        CountDownLatch cdl = new CountDownLatch(param.getContents().size());
        List<Future<Content>> futures = new ArrayList<>();
        Map<String, String> titleMd5Map = new HashMap<>();
        Map<String, ArticleSensitive> articleSensitiveMap = new HashMap<>();
        List<String> md5List = new ArrayList<>();
        for (Content content : param.getContents()) {
            String md5 = Md5Util.encoderByMd5(content.getTitle());
            md5List.add(md5);
            titleMd5Map.put(content.getTitle(), md5);
        }
        List<ArticleSensitive> articleSensitiveList = getArticleSensitive(md5List);
        if (CollectionUtil.isNotEmpty(articleSensitiveList)) {
            articleSensitiveMap = articleSensitiveList.stream().collect(Collectors.toMap(ArticleSensitive::getMd5, o -> o));
        }

        for (Content content : param.getContents()) {
            Map<String, ArticleSensitive> finalArticleSensitiveMap = articleSensitiveMap;
            Future<Content> future = pool.submit(() -> {
                try {
//                    boolean isSensitive = articleSensitiveRemoteService.articleSensitive(content.getTitle(),
//                            titleMd5Map,
//                            finalArticleSensitiveMap);
//                    if (isSensitive) {
//                        content.setFilterReason("安全违规");
//                    } else
                    if (TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), UnSafeTitles)) {
                        content.setFilterReason("安全违规");
                    }
                    return content;
                } finally {
                    cdl.countDown();
                }
            });
            futures.add(future);
        }
        try {
            cdl.await(5000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("filter error", e);
            return null;
        }

        for (Future<Content> f : futures) {
            try {
                Content content = f.get();
                if (StringUtils.hasText(content.getFilterReason())) {
                    filterContents.add(content);
                } else {
                    result.add(content.getId());
                }
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        log.info("SensitiveStrategy cost:{}", System.currentTimeMillis() - start);
        return filterResult;
    }

    public List<ArticleSensitive> getArticleSensitive(List<String> md5List) {
        return articleSensitiveRepository.findAllById(md5List);
    }

}
