package com.tzld.longarticle.recommend.server.service.recommend.rank;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.enums.recommend.RankStrategyEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.ServiceBeanFactory;
import com.tzld.longarticle.recommend.server.service.recommend.rank.strategy.FwhColdStartRankStrategy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;

import static com.tzld.longarticle.recommend.server.common.constant.SceneConstants.FWH_COLD_START;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RankService {

    public RankResult rank(RankParam param) {
        RankStrategy strategy = getRankStrategy(param);
        return strategy.rank(param);
    }

    private RankStrategy getRankStrategy(RankParam param) {

        switch (param.getScene()) {
            case FWH_COLD_START:
                return ServiceBeanFactory.getBean(FwhColdStartRankStrategy.class);
            default:
                RankStrategyEnum rankStrategyEnum = RankStrategyEnum.from(param.getStrategy());
                return (RankStrategy) ServiceBeanFactory.getBeanByName(rankStrategyEnum.getTaskExecutorName());
        }

    }

    public static void printSortLog(String strategy, String accountName, List<Content> contentList) {
        JSONArray jsonArray = new JSONArray();
        for (Content content : contentList) {
            JSONObject obj = new JSONObject();
            obj.put("id", content.getId());
            obj.put("title", content.getTitle());
            obj.put("score", content.getScore());
            obj.put("scoreMap", content.getScoreMap());
            jsonArray.add(obj);
        }
        log.info("{} 账号名称 {} 头条评分结果 {}", strategy, accountName, JSONObject.toJSONString(jsonArray));
    }
}
