package com.tzld.longarticle.recommend.server.service.recommend.rank.strategy;

import com.tzld.longarticle.recommend.server.common.enums.recommend.ScoreStrategyEnum;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.service.recommend.config.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankItem;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.recommend.rank.RankStrategy;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreParam;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreResult;
import com.tzld.longarticle.recommend.server.service.recommend.score.ScoreService;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class LateRankStrategy implements RankStrategy {

    @Autowired
    private ScoreService scoreService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    public RankResult rank(RankParam param) {
        List<Content> result = new ArrayList<>();
        String[] contentPools = accountContentPoolConfigService.getContentPools(param.getAccountName());
        ScoreParam scoreParam = RankStrategy.convertToScoreParam(param);
        scoreParam.setContents(scoreParam.getContents().stream().filter(
                c -> contentPools[2].equals(c.getContentPoolType())).collect(Collectors.toList()));
        ScoreResult scoreResult = scoreService.score(RankStrategy.convertToScoreParam(param));

        Map<String, Map<String, Double>> scoreMap = scoreResult.getScoreMap();

        List<RankItem> items = CommonCollectionUtils.toList(scoreParam.getContents(), c -> {
            RankItem item = new RankItem();
            item.setContent(c);
            c.setScoreMap(scoreMap.get(c.getId()));
            item.setScoreMap(scoreMap.get(c.getId()));
            double score = 0.0;
            if (contentPools[2].equals(item.getContent().getContentPoolType())) {
                score = item.getScore(ScoreStrategyEnum.SIMILARITY.value())
                        + item.getScore(ScoreStrategyEnum.CATEGORY.value())
                        + item.getScore(ScoreStrategyEnum.ACCOUNT_PRE_DISTRIBUTE.value())
                        + item.getScore(ScoreStrategyEnum.PUBLISH_TIMES.value())
                        + item.getScore(ScoreStrategyEnum.CRAWLER_DAYS_DECREASE_STRATEGY.value())
                        + item.getScore(ScoreStrategyEnum.FLOW_CTL_DECREASE.value());
            }
            c.setScore(score);
            item.setScore(score);
            return item;
        });

        // 1 排序
        Collections.sort(items, (o1, o2) -> -Double.compare(o1.getScore(), o2.getScore()));
        // 2 相似去重
        List<Content> contents = CommonCollectionUtils.toList(items, RankItem::getContent);

        // 3 文章按照内容池分组
        Map<String, List<Content>> contentMap = new HashMap<>();
        for (Content c : contents) {
            List<Content> data = contentMap.computeIfAbsent(c.getContentPoolType(), k -> new ArrayList<>());
            data.add(c);
        }

        // 全部使用3-8内容池
        List<Content> pool = contentMap.get(contentPools[2]);
        if (CollectionUtils.isNotEmpty(pool)) {
            result.addAll(pool.subList(0, Math.min(pool.size(), param.getSize())));
        }

        return new RankResult(result);
    }

}
