package com.tzld.longarticle.recommend.server;

import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.fastjson.JSONObject;
import com.tencentcloudapi.tms.v20201229.models.TextModerationResponse;
import com.tzld.longarticle.recommend.server.model.dto.Content;
import com.tzld.longarticle.recommend.server.remote.AIGCRemoteService;
import com.tzld.longarticle.recommend.server.remote.ArticleSensitiveRemoteService;
import com.tzld.longarticle.recommend.server.repository.crawler.AccountAvgInfoRepository;
import com.tzld.longarticle.recommend.server.model.entity.crawler.AccountAvgInfo;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.recommend.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.service.recommend.recall.RecallParam;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DuplicateKeyException;

import javax.annotation.Resource;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;
import java.util.Set;

@SpringBootTest(classes = Application.class)
@Slf4j
public class FilterStrategyTest {

    @Resource
    FilterStrategy badStrategy;
    @Resource
    FilterStrategy historyTitleStrategy;
    @Resource
    AIGCRemoteService aigcRemoteService;
    @Resource
    ArticleSensitiveRemoteService articleSensitiveRemoteService;
    @Resource
    private AccountAvgInfoRepository accountAvgInfoRepository;
//    @Test
//    public void badStrategyTest() {
//        FilterParam param = new FilterParam();
//        List<Content> contentList = new ArrayList<>();
//        Content content0 = new Content();
//        content0.setId("0");
//        content0.setTitle("中国第一位小品女皇，因一句台词识破潜伏间谍");
//        contentList.add(content0);
//        Content content1 = new Content();
//        content1.setId("1");
//        content1.setTitle("终身未娶拉扯张长大28年，女孩回报方式看哭众人");
//        contentList.add(content1);
//        param.setContents(contentList);
//        FilterResult result = badStrategy.filter(param);
//        System.out.println(JSONObject.toJSONString(result));
//    }
    @Test
    public void historyTitleStrategyTest() {
        // accountId:20231225071712749835955 accountName:奇趣百味生活 planId:20240627143100701713157
        RecallParam param = new RecallParam();
        param.setAccountName("票圈最新消息");
        param.setAccountId("20231212144543068928298");
        param.setPlanId("20240724092832149199870");
        List<Content> content = aigcRemoteService.getAllContent(param);;
        FilterParam filterParam = new FilterParam();
        filterParam.setAccountName(param.getAccountName());
        filterParam.setContents(content);
        FilterResult result = historyTitleStrategy.filter(filterParam);
        System.out.println(JSONObject.toJSONString(result));
    }

    @Test
    public void dealHistoryArticleSensitiveJSON() {

        String folderPath = "/Users/wangyunpeng/Downloads/moderation.tar/moderation";

        File folder = new File(folderPath);
        File[] listOfFiles = folder.listFiles((dir, name) -> name.endsWith(".json"));

        if (listOfFiles != null) {
            for (File file : listOfFiles) {
                try {
                    String content = new String(Files.readAllBytes(file.toPath()), StandardCharsets.UTF_8);
                    JSONObject jsonObject = JSONObject.parseObject(content);
                    TextModerationResponse data = jsonObject.getObject("data", TextModerationResponse.class);
                    String title = jsonObject.getString("text");
                    String md5 = Md5Util.encoderByMd5(title);
                    articleSensitiveRemoteService.insertArticleSensitive(md5, title, data.getSuggestion(),
                            data.getLabel(), data.getSubLabel(), JSONObject.toJSONString(data));
                    log.info("readFile fileName:{} content:{}", file.getName(), jsonObject.toJSONString());
                } catch (DuplicateKeyException ignore) {
                } catch (Exception e) {
                    log.error("readFileError fileName:{}",file.getName(), e);
                }
            }
        } else {
            log.error("No JSON files found in the folder.");
        }
    }

    @Test
    void AccountAvgInfoTest() {
        String cardJSON = ResourceUtil.readUtf8Str("file/AccountInfo.json");
        JSONObject accountInfo = JSONObject.parseObject(cardJSON);
        Set<String> keys = accountInfo.keySet();
        for (String key : keys) {
            AccountAvgInfo info = accountInfo.getObject(key, AccountAvgInfo.class);
            accountAvgInfoRepository.save(info);
        }
    }
}
